/*
 *
 * 'Driver' for the 'Error' LED on the Soekris Net45[20]1
 *
 */

#include <sys/param.h>
#include <sys/device.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <sys/conf.h>
#include <sys/systm.h>

#include <arch/i386/mmcr/skledvar.h>
#include <arch/i386/mmcr/mmcrreg.h>

#define LED_BIT		0x0200

static int
skledmatch(struct device *parent, struct cfdata *match, void *aux);

static void
skledattach(struct device *parent, struct device *self, void *aux);

cdev_decl(skled);

struct skled_softc {
	struct device 	sc_dev;
	/* the MMCR base address after we've mapped it in */
	caddr_t		mmcr_base;
	u_char		attached;
};

struct cfattach skled_ca = {
	sizeof(struct skled_softc), skledmatch, skledattach
};

extern struct cfdriver skled_cd;

extern caddr_t mmcr_base;

static int
skledmatch(struct device *parent, struct cfdata *match, void *aux)
{
	struct skled_attach_args *skaa = aux;

	if (strcmp(skaa->skaa_busname, "skled") != 0)
		return(0);

	return(1);
}

static void
skledattach(struct device *parent, struct device *self, void *aux)
{
	struct skled_softc *sc = (void *)self;

	/* should already be set for us */
	sc->mmcr_base = mmcr_base;

	printf(": LED %s\n", 
		(*(u_int32_t *)(sc->mmcr_base + MMCR_GPIO_DATA) &
		LED_BIT) ?  "On" : "Off");
	
	sc->attached = 1;
	
	/* If the led is already on, switch if off */
	*(u_int32_t *)(sc->mmcr_base + MMCR_GPIO_CLEAR) = LED_BIT;
}

int
skledopen(dev_t dev, int flags, int fmt, struct proc *p)
{
	struct skled_softc *sc;
	int unit = minor(dev);

	if (unit != 0)
		return (ENXIO);
	
	sc = skled_cd.cd_devs[unit];

	if (!sc)
		return ENXIO;

	if (!sc->attached)
		return(ENXIO);

	if (*(u_int32_t *)(sc->mmcr_base + MMCR_GPIO_DATA) & LED_BIT)
	{
		*(u_int32_t *)(sc->mmcr_base + MMCR_GPIO_CLEAR) = LED_BIT;
	} else {
		*(u_int32_t *)(sc->mmcr_base + MMCR_GPIO_SET) = LED_BIT;
	}

	/* Always succeeds. */
	return (0);
}

int
skledclose(dev_t dev, int flags, int fmt, struct proc *p)
{
	/* Always succeeds. */
	return(0);
}


int
skledioctl(dev_t dev, u_long cmd, caddr_t data, int flag, struct proc *p)
{
	printf("skledioctl : got cmd 0x%08lx\n", cmd);
	return EINVAL;
}

